

import DocParser from '../util/docparser.js';
import { SimpleList } from '../../lib/index.js';
import { ExampledView } from './example.js';
SimpleList.register();


import rawcode from '!raw-loader!../../lib/components/list.js';

let __docs = undefined;

const code = `this.itemTemplate = (item) => { 
  return this.builder\`<div class="row"><div class="colorblock" style="background-color: \${item.value};"></div><span>\${item.text}</span></div>\`};

this.listStyles = [ this.cssBuilder\`
  .row {
    display: flex;
    flex-direction: row;
    align-items: center;
  }
  .colorblock {
    width: 20px;
    height: 0.75em;
    margin: 0px 1em;
    border: 1px solid grey;
  }\` ]`;

const example = `<sc-list 
  select="multi" 
  modelName="colours"
></sc-list>`;



class ListsView extends ExampledView {

  static title() {
    return "Lists";
  }

  static propertyElements() {
    return [ 'sc-list' ];
  }
  static propertyTables() {
    return [ 'list-properties' ];
  }

  static varElements() {
    return [ 'sc-list' ];
  }

  static varTables() {
    return [ 'list-styles' ]
  }

  static methodNames( ) {
    return [ 'total', 'length', 'applySort', 'applyFilters', 'getDisplayedItems', 'getDisplayedIDs', 'clear' ];
  }

  static eventNames( ) {
    return [ 'updated' ];
  }

  static getDocs() {
    if ( __docs == undefined )
      __docs = DocParser.parseJs( rawcode );
    return __docs
  }


  static get tileTemplate() {
    return (item) => {
      return this.builder`
        <h4>${item.name}</h4>
        <span>${item.email}</span>
        <span>${item.comment}</span>
      `
    };
  }

  static get listTemplate() {
    return (item) => {
      return this.builder`
        <span>${item.name}</span>
      `
    }
  }

  static get colourTemplate() {
    return (item) => {
      return this.builder`
        <div style="display: flex; align-items: center; width: 100%;"><div style="width: 20px; height: 0.75em; display: inline-block; margin: 0px 1em; background-color: ${item.value}; border: 1px solid grey;"></div><span>${item.text}</span></div>`
    }
  }


  static get tileStyles() {
    return this.cssBuilder`
      .sc-list-item {
        width: 250px;
        height: 100px;
        flex-direction: column;
        align-items: start !important;
        background-color: var( --theme-third );
        border-radius: var( --list-border-radius );
        overflow-wrap: break-word;
        overflow: hidden;
        margin: var( --list-margin );
        word-break: break-word;
      }
      .sc-list-item h4 {
        margin-bottom: 10px;
      }
    `
  }


  static build() {
    return this.builder`
      <div id="lists-page">
        <h2>Lists</h2>

        <div>
          <h3 class="lined">Description</h3>
          <p>The <code>&lt;sc-list&gt;</code> component is a user interactable element to display a list of information, generated by a template builder function. It is able to accommodate basic lists, tiles or cards of content, and provides functionality to select items within the list.</p>
          <h3 class="lined">Try it out</h3>
          ${ExampledView.build( example, code )}
        </div>


        <div class="properties">
          <h3 class="lined">Properties</h3>
          <h4><code>&lt;sc-list&gt;</code></h4>
          <sc-table
              id="list-properties"
              scrollable
              .fieldDefinitions=${this.propFields}
          >
          </sc-table>
        </div>


        <div class="events">
          <h3 class="lined">Events</h3>
          ${this.buildEvents( 'SimpleList' )}
        </div>


        <div class="methods">
          <h3 class="lined">Methods</h3>
          ${this.buildMethods( 'SimpleList' )}
        </div>


        <div class="variables">
          <h3 class="lined">CSS Variables</h3>
          <p></p>
          <sc-table
              id="list-styles"
              scrollable
              .fieldDefinitions=${this.varFields}
          >
          </sc-table>
        </div>


        <div id="examples" class="examples" @click=${this.chooseExample.bind( this )}>
          <h3 class="lined">Examples</h3>
          <p>Select and try out any example:<sc-button id="choose-button" type="toggle" label="Choose example..." @click=${this.setChooseMode}></sc-button></p>

          <div class="example-row">

            <div class="example">
              <h4>Simple text list</h4>
              <sc-list 
                modelname="example"
                .itemTemplate=${this.listTemplate}>
              </sc-list>
            </div>

            <div class="example">
              <h4>Selectable text list</h4>
              <sc-list 
                select="multi"
                modelname="example"
                .itemTemplate=${this.listTemplate}>
              </sc-list>
            </div>

            <div class="example">
              <h4>Selectable rich list</h4>
              <sc-list 
                select="multi"
                modelname="colours"
                .itemTemplate=${this.colourTemplate}>
              </sc-list>
            </div>
          </div>


          <h4>Simple tiles (responsive, resizable)</h4>
          <div class="example-row" style="min-height: 400px;">
            <div class="example" style="resize: both; overflow: hidden; height: 340px; width: 650px;">
              <sc-list 
                fillwrap
                select="multi"
                modelname="example"
                @item-dbl-click=${this.dblClick}
                .itemTemplate=${this.tileTemplate} 
                .listStyles=${this.tileStyles}>
              </sc-list>
            </div>
          </div>

        </div>
      </div>
    `
  }


  static dblClick( ev ) {
    console.log( "Item double-clicked with", ev );
  }


}


export { ListsView };

