

import DocParser from '../util/docparser.js';
import { ExampledView } from './example.js';


import rawcode from '!raw-loader!../../lib/models/integrated.js';

let __docs = undefined;



class ModelIntegratedView extends ExampledView {

  static methodNames( ) {
    return [ 'constructor', 'setup', 'load', 'reload', 'fetch', 'refetch', 'clear', 'handleNotify', 'handleDisconnect' ];
  }

  static getDocs() {
    if ( __docs == undefined )
      __docs = DocParser.parseJs( rawcode );
    return __docs
  }



  static build() {
    return this.builder`
      <div id="model-integrated-page">
        <h2>Integrated Models</h2>
        <h3 class="lined">Description</h3>
        <p>The <code>IntegratedModel</code> class is a mixin that augments the <code>RemoteModel</code> to be a partial cache model. To achieve this, the model does not <code>load</code> records from the remote source, rather records are fetched from the remote source upon a fetch on the method. Additionally, to ensure that only required records are fetched, sorting and filtering is attended server side, hence the name.</p>

        <p>In addition to being a partial cache model, the <code>IntegratedModel</code> also allows the use of a channel-based WebSocket connection to maintain consistency with the remote data source.</p> 

        
        <h3 class="lined">Data Consistency</h3>
        <p>Consistency in the data of the <code>IntegratedModel</code> is acheived through a WebSocket connection where the data source will publish changes that may impact the records of this model. To make allowances for multiple data source clients, and multiple data models requiring updates for data sources, a publishing / subscribing channel based WebSocket implementation is used. To use this functionality, a <code>wsChannel</code> and <code>websock</code> URL parameter is required.</p>

        <p>For data sources implementing WebSocket server functionality, a flexible ws based utility library class <code>ChannelSock</code> is available. This utility provides the channel pub/sub functionality with a simple API to notify of changes to subscribed clients. See inline documentation to implement this in your project.</p>


        <h3 class="lined">Filtering & Sorting</h3>
        <p>For <code>IntegratedModel</code> models, filtering is required to be completed by the server. To acheieve this, filter rules from the client are serialised and sent to the server with the fetch request. To assist with this, a utility class <code>QueryTranslate</code> is available in this library to translate the application filter structure into a SQL constraint clause.</p>

        <p>Similar to filtering, sorting for <code>IntegratedModel</code> is required on the server. The sorting structure is sent with each fetch request. Translating the application sorting structure into a form for the data source is generally not complex.</p>
        


        <div class="methods">
          <h3 class="lined">Methods</h3>
          <p></p>
          ${this.buildMethods( 'SimpleIntegratedCollection' )}
        </div>


      </div>
    `
  }


}

export { ModelIntegratedView };

